<?php
namespace AIOSEO\Plugin\Common\Migration;

// phpcs:disable WordPress.Arrays.ArrayDeclarationSpacing.AssociativeArrayFound

use AIOSEO\Plugin\Common\Models;

/**
 * Migrates the post meta from V3.
 *
 * @since 4.0.0
 */
class Meta {

	/**
	 * Migrates the plugin meta data.
	 *
	 * @since 4.0.0
	 *
	 * @return void
	 */
	public function migrateMeta() {
		try {
			$actions = as_get_scheduled_actions( [
				'hook'     => 'aioseo_migrate_post_meta',
				'status'   => \ActionScheduler_Store::STATUS_PENDING,
				'per_page' => 1
			]);

			if ( $actions ) {
				return;
			}

			if ( ! get_transient( 'aioseo_v3_migration_in_progress_posts' ) ) {
				set_transient( 'aioseo_v3_migration_in_progress_posts', time(), WEEK_IN_SECONDS );
			}

			as_schedule_single_action( time(), 'aioseo_migrate_post_meta' );
		} catch ( \RuntimeException $e ) {
			// Do nothing.
		}

		$this->migratePostMeta();
	}

	/**
	 * Migrates the post meta data from V3.
	 *
	 * @since 4.0.0
	 *
	 * @return void
	 */
	public function migratePostMeta() {
		$postsPerAction  = 100;
		$publicPostTypes = implode( '", "', aioseo()->helpers->getPublicPostTypes( true ) );

		$postsToMigrate = aioseo()->db
			->start( 'posts' . ' as p' )
			->select( 'p.ID' )
			->leftJoin( 'aioseo_posts as ap', '`p`.`ID` = `ap`.`post_id`' )
			->where( 'ap.post_id', null )
			->whereRaw( "( p.post_type IN ( \"$publicPostTypes\" ) )" )
			->limit( $postsPerAction )
			->run()
			->result();

		if ( ! $postsToMigrate || ! count( $postsToMigrate ) ) {
			delete_transient( 'aioseo_v3_migration_in_progress_posts' );
			return;
		}

		$excludedPosts        = [];
		$sitemapExcludedPosts = [];

		foreach ( $postsToMigrate as $post ) {
			$postMeta = aioseo()->db
				->start( 'postmeta' . ' as pm' )
				->select( 'pm.meta_key, pm.meta_value' )
				->where( 'pm.post_id', $post->ID )
				->whereRaw( '`pm`.`meta_key` LIKE "_aioseop_%"' )
				->run()
				->result();

			$mappedMeta = [
				'_aioseop_title'              => 'title',
				'_aioseop_description'        => 'description',
				'_aioseop_custom_link'        => 'canonical_url',
				'_aioseop_sitemap_exclude'    => '',
				'_aioseop_disable'            => '',
				'_aioseop_noindex'            => 'robots_noindex',
				'_aioseop_nofollow'           => 'robots_nofollow',
				'_aioseop_sitemap_priority'   => 'priority',
				'_aioseop_sitemap_frequency'  => 'frequency',
				'_aioseop_keywords'           => 'keywords',
				'_aioseop_opengraph_settings' => ''
			];

			$meta = [
				'post_id' => $post->ID,
			];

			if ( ! $postMeta || ! count( $postMeta ) ) {
				$post = Models\Post::getPost( $post->ID );
				$post->set( $meta );
				$post->save();
				continue;
			}

			$post = get_post( $post->ID );
			foreach ( $postMeta as $record ) {
				$name  = $record->meta_key;
				$value = $record->meta_value;

				if ( ! in_array( $name, array_keys( $mappedMeta ), true ) ) {
					continue;
				}

				switch ( $name ) {
					case '_aioseop_description':
						$meta[ $mappedMeta[ $name ] ] = aioseo()->helpers->sanitizeOption( aioseo()->migration->helpers->macrosToSmartTags( $value ) );
						break;
					case '_aioseop_title':
						if ( ! empty( $value ) ) {
							$meta[ $mappedMeta[ $name ] ] = $this->getTitleValue( $post, $value );
						}
						break;
					case '_aioseop_sitemap_exclude':
						if ( empty( $value ) ) {
							break;
						}
						$sitemapExcludedPosts[] = $post->ID;
						break;
					case '_aioseop_disable':
						if ( empty( $value ) ) {
							break;
						}
						$excludedPosts[] = $post->ID;
						break;
					case '_aioseop_noindex':
					case '_aioseop_nofollow':
						if ( 'on' === (string) $value ) {
							$meta['robots_default']       = false;
							$meta[ $mappedMeta[ $name ] ] = true;
						} elseif ( 'off' === (string) $value ) {
							$meta['robots_default'] = false;
						}
						break;
					case '_aioseop_keywords':
						$meta[ $mappedMeta[ $name ] ] = aioseo()->migration->helpers->oldKeywordsToNewKeywords( $value );
						break;
					case '_aioseop_opengraph_settings':
						$meta += $this->convertOpenGraphMeta( $value );
						break;
					default:
						$meta[ $mappedMeta[ $name ] ] = esc_html( wp_strip_all_tags( strval( $value ) ) );
						break;
				}
			}

			// If the post didn't have a title set at meta level, we still need to run it through the format.
			if ( ! in_array( 'title', array_keys( $meta ), true ) ) {
				$meta['title'] = $this->getTitleValue( $post );
			}

			$aioseoPost = Models\Post::getPost( $post->ID );
			$aioseoPost->set( $meta );
			$aioseoPost->save();
		}

		$this->migrateAdditionalPostMeta( $post->ID );

		if ( count( $excludedPosts ) ) {
			$deprecatedOptions = aioseo()->internalOptions->internal->deprecatedOptions;
			if ( ! in_array( 'excludePosts', $deprecatedOptions, true ) ) {
				array_push( $deprecatedOptions, 'excludePosts' );
				aioseo()->internalOptions->internal->deprecatedOptions = $deprecatedOptions;
			}

			$posts = aioseo()->options->deprecated->searchAppearance->advanced->excludePosts;

			foreach ( $excludedPosts as $id ) {
				if ( ! intval( $id ) ) {
					continue;
				}
				$post = get_post( $id );
				if ( ! is_object( $post ) ) {
					continue;
				}
				$excludedPost        = new \stdClass();
				$excludedPost->type  = $post->post_type;
				$excludedPost->value = $post->ID;
				$excludedPost->label = $post->post_title;
				$excludedPost->link  = get_permalink( $post );

				$posts[] = wp_json_encode( $excludedPost );
			}
			aioseo()->options->deprecated->searchAppearance->advanced->excludePosts = $posts;
		}

		if ( count( $sitemapExcludedPosts ) ) {
			aioseo()->options->sitemap->general->advancedSettings->enable = true;

			$posts = aioseo()->options->sitemap->general->advancedSettings->excludePosts;
			foreach ( $sitemapExcludedPosts as $id ) {
				if ( ! intval( $id ) ) {
					continue;
				}
				$post = get_post( $id );
				if ( ! is_object( $post ) ) {
					continue;
				}
				$excludedPost        = new \stdClass();
				$excludedPost->type  = $post->post_type;
				$excludedPost->value = $post->ID;
				$excludedPost->label = $post->post_title;
				$excludedPost->link  = get_permalink( $post );

				$posts[] = wp_json_encode( $excludedPost );
			}
			aioseo()->options->sitemap->general->advancedSettings->excludePosts = $posts;
		}

		if ( count( $postsToMigrate ) === $postsPerAction ) {
			try {
				$actions = as_get_scheduled_actions( [
					'hook'     => 'aioseo_migrate_post_meta',
					'status'   => \ActionScheduler_Store::STATUS_PENDING,
					'per_page' => 1
				]);

				if ( $actions ) {
					return;
				}
				as_schedule_single_action( time(), 'aioseo_migrate_post_meta' );
			} catch ( \RuntimeException $e ) {
				// Do nothing.
			}
		} else {
			delete_transient( 'aioseo_v3_migration_in_progress_posts' );
		}
	}

	/**
	 * Migrates additional post meta data.
	 *
	 * @since 4.0.2
	 *
	 * @param  int  $postId The post ID.
	 * @return void
	 */
	protected function migrateAdditionalPostMeta( $postId ) {
		return $postId;
	}

	/**
	 * Maps the old Open Graph meta to the social meta columns in V4.
	 *
	 * @since 4.0.0
	 *
	 * @param  array $ogMeta The old V3 Open Graph meta.
	 * @return array $meta   The mapped meta.
	 */
	public function convertOpenGraphMeta( $ogMeta ) {
		$ogMeta = maybe_unserialize( $ogMeta );

		if ( ! is_array( $ogMeta ) ) {
			return [];
		}

		$mappedSocialMeta = [
			'aioseop_opengraph_settings_title'             => 'og_title',
			'aioseop_opengraph_settings_desc'              => 'og_description',
			'aioseop_opengraph_settings_customimg'         => 'og_image_custom_url',
			'aioseop_opengraph_settings_imagewidth'        => 'og_image_width',
			'aioseop_opengraph_settings_imageheight'       => 'og_image_height',
			'aioseop_opengraph_settings_video'             => 'og_video',
			'aioseop_opengraph_settings_videowidth'        => 'og_video_width',
			'aioseop_opengraph_settings_videoheight'       => 'og_video_height',
			'aioseop_opengraph_settings_category'          => 'og_object_type',
			'aioseop_opengraph_settings_section'           => 'og_article_section',
			'aioseop_opengraph_settings_tag'               => 'og_article_tags',
			'aioseop_opengraph_settings_setcard'           => 'twitter_card',
			'aioseop_opengraph_settings_customimg_twitter' => 'twitter_image_custom_url',
		];

		$meta = [];
		foreach ( $ogMeta as $name => $value ) {
			if ( ! in_array( $name, array_keys( $mappedSocialMeta ), true ) ) {
				continue;
			}

			switch ( $name ) {
				case 'aioseop_opengraph_settings_desc':
				case 'aioseop_opengraph_settings_title':
					$meta[ $mappedSocialMeta[ $name ] ] = aioseo()->helpers->sanitizeOption( aioseo()->migration->helpers->macrosToSmartTags( $value ) );
					break;
				case 'aioseop_opengraph_settings_customimg':
					$value = strval( $value );
					if ( empty( $value ) ) {
						break;
					}

					$meta['og_image_type']              = 'custom_image';
					$meta[ $mappedSocialMeta[ $name ] ] = strval( $value );
					break;
				case 'aioseop_opengraph_settings_video':
					$meta[ $mappedSocialMeta[ $name ] ] = esc_url( $value );
					break;
				case 'aioseop_opengraph_settings_customimg_twitter':
					$value = strval( $value );
					if ( empty( $value ) ) {
						break;
					}
					$meta['twitter_image_type']         = 'custom_image';
					$meta['twitter_use_og']             = false;
					$meta[ $mappedSocialMeta[ $name ] ] = strval( $value );
					break;
				case 'aioseop_opengraph_settings_imagewidth':
				case 'aioseop_opengraph_settings_imageheight':
				case 'aioseop_opengraph_settings_videowidth':
				case 'aioseop_opengraph_settings_videoheight':
					$value = intval( $value );
					if ( ! $value || $value <= 0 ) {
						break;
					}
					$meta[ $mappedSocialMeta[ $name ] ] = $value;
					break;
				case 'aioseop_opengraph_settings_tag':
					$meta[ $mappedSocialMeta[ $name ] ] = aioseo()->migration->helpers->oldKeywordsToNewKeywords( $value );
					break;
				default:
					$meta[ $mappedSocialMeta[ $name ] ] = esc_html( strval( $value ) );
					break;
			}
		}

		return $meta;
	}

	/**
	 * Returns the title as it was in V3.
	 *
	 * @since 4.0.0
	 *
	 * @param  int    $post     The post object.
	 * @param  string $seoTitle The old SEO title.
	 * @return string           The title.
	 */
	protected function getTitleValue( $post, $seoTitle = '' ) {
		if ( ! is_object( $post ) ) {
			return '';
		}

		$titleFormat = '%post_title% | %site_title%';
		if ( isset( aioseo()->migration->oldOptions[ "aiosp_{$post->post_type}_title_format" ] ) ) {
			$titleFormat = aioseo()->migration->oldOptions[ "aiosp_{$post->post_type}_title_format" ] ? aioseo()->migration->oldOptions[ "aiosp_{$post->post_type}_title_format" ] : $titleFormat;
		}

		return aioseo()->helpers->sanitizeOption( aioseo()->migration->helpers->macrosToSmartTags( $seoTitle ) );
	}
}