<?php
namespace AIOSEO\Plugin\Pro\Migration;

// phpcs:disable WordPress.Arrays.ArrayDeclarationSpacing.AssociativeArrayFound

use AIOSEO\Plugin\Common\Migration as CommonMigration;
use AIOSEO\Plugin\Pro\Models;
use AIOSEO\Plugin\Extend\VideoSitemap;

/**
 * Migrates the term meta from V3.
 *
 * @since 4.0.0
 */
class Meta extends CommonMigration\Meta {

	/**
	 * Instance of the Video class of the Video Sitemap.
	 *
	 * @since 4.0.2
	 *
	 * @var Video
	 */
	private $videoSitemap = null;

	/**
	 * Class constructor.
	 *
	 * @since 4.0.2
	 */
	public function __construct() {
		add_action( 'init', [ $this, 'instantiateVideoSitemap' ] );
	}

	/**
	 * Instantiates the Video Sitemap video class if the addon is installed and active.
	 *
	 * @since 4.0.2
	 *
	 * @return void
	 */
	public function instantiateVideoSitemap() {
		include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		if ( is_plugin_active( aioseo()->addons->getAddon( 'aioseo-video-sitemap' )->basename ) && aioseo()->license->isAddonAllowed( 'aioseo-video-sitemap' ) ) {
			$this->videoSitemap = new VideoSitemap\Video;
		}
	}

	/**
	 * Migrates additional post meta data.
	 *
	 * @since 4.0.2
	 *
	 * @param  int  $postId The post ID.
	 * @return void
	 */
	protected function migrateAdditionalPostMeta( $postId ) {
		if ( $this->videoSitemap ) {
			$post = get_post( $postId );
			$this->videoSitemap->scanPost( $post );
		}
	}

	/**
	 * Migrates the plugin meta data.
	 *
	 * @since 4.0.0
	 *
	 * @return void
	 */
	public function migrateMeta() {
		parent::migrateMeta();

		try {
			$actions = as_get_scheduled_actions( [
				'hook'     => 'aioseo_migrate_term_meta',
				'status'   => \ActionScheduler_Store::STATUS_PENDING,
				'per_page' => 1
			]);

			if ( $actions ) {
				return;
			}

			if ( ! get_transient( 'aioseo_v3_migration_in_progress_terms' ) ) {
				set_transient( 'aioseo_v3_migration_in_progress_terms', time(), WEEK_IN_SECONDS );
			}

			as_schedule_single_action( time(), 'aioseo_migrate_term_meta' );
		} catch ( \RuntimeException $e ) {
			// Do nothing.
		}

		$this->migrateTermMeta();
	}

	/**
	 * Migrates the term meta data from V3.
	 *
	 * @since 4.0.0
	 *
	 * @return void
	 */
	public function migrateTermMeta() {
		$termsPerAction   = 20;
		$publicTaxonomies = implode( '", "', aioseo()->helpers->getPublicTaxonomies( true ) );

		$termsToMigrate = aioseo()->db
			->start( 'terms' . ' as t' )
			->select( 't.term_id' )
			->leftJoin( 'aioseo_terms as at', '`t`.`term_id` = `at`.`term_id`' )
			->leftJoin( 'term_taxonomy as tt', '`t`.`term_id` = `tt`.`term_id`' )
			->where( 'at.term_id', null )
			->whereRaw( "( tt.taxonomy IN ( \"$publicTaxonomies\" ) )" )
			->limit( $termsPerAction )
			->run()
			->result();

		if ( ! $termsToMigrate || ! count( $termsToMigrate ) ) {
			delete_transient( 'aioseo_v3_migration_in_progress_terms' );
			return;
		}

		$excludedTerms        = [];
		$sitemapExcludedTerms = [];

		foreach ( $termsToMigrate as $term ) {
			$termMeta = aioseo()->db
				->start( 'termmeta' . ' as tm' )
				->select( '`tm`.`meta_key`, `tm`.`meta_value`' )
				->where( 'tm.term_id', $term->term_id )
				->whereRaw( '`tm`.`meta_key` LIKE "_aioseop_%"' )
				->run()
				->result();

			$mappedMeta = [
				'_aioseop_title'              => 'title',
				'_aioseop_description'        => 'description',
				'_aioseop_custom_link'        => 'canonical_url',
				'_aioseop_sitemap_exclude'    => '',
				'_aioseop_disable'            => '',
				'_aioseop_noindex'            => 'robots_noindex',
				'_aioseop_nofollow'           => 'robots_nofollow',
				'_aioseop_sitemap_priority'   => 'priority',
				'_aioseop_sitemap_frequency'  => 'frequency',
				'_aioseop_keywords'           => 'keywords',
				'_aioseop_opengraph_settings' => ''
			];

			$meta = [
				'term_id' => $term->term_id,
			];

			if ( ! $termMeta || ! count( $termMeta ) ) {
				$term = Models\Term::getTerm( $term->term_id );
				$term->set( $meta );
				$term->save();
				continue;
			}

			$term = get_term( $term->term_id );
			foreach ( $termMeta as $record ) {
				$name  = $record->meta_key;
				$value = $record->meta_value;

				if ( ! in_array( $name, array_keys( $mappedMeta ), true ) ) {
					continue;
				}

				switch ( $name ) {
					case '_aioseop_description':
						$meta[ $mappedMeta[ $name ] ] = aioseo()->helpers->sanitizeOption( aioseo()->migration->helpers->macrosToSmartTags( $value ) );
						break;
					case '_aioseop_title':
						if ( ! empty( $value ) ) {
							$meta[ $mappedMeta[ $name ] ] = $this->getTitleValue( $term, $value );
						}
						break;
					case '_aioseop_sitemap_exclude':
						if ( empty( $value ) ) {
							break;
						}
						$sitemapExcludedTerms[] = $term->term_id;
						break;
					case '_aioseop_disable':
						if ( empty( $value ) ) {
							break;
						}
						$excludedTerms[] = $term->term_id;
						break;
					case '_aioseop_noindex':
					case '_aioseop_nofollow':
						if ( 'on' === (string) $value ) {
							$meta['robots_default']       = false;
							$meta[ $mappedMeta[ $name ] ] = true;
						} elseif ( 'off' === (string) $value ) {
							$meta['robots_default'] = false;
						}
						break;
					case '_aioseop_keywords':
						$meta[ $mappedMeta[ $name ] ] = aioseo()->migration->helpers->oldKeywordsToNewKeywords( $value );
						break;
					case '_aioseop_opengraph_settings':
						$meta += $this->convertOpenGraphMeta( $value );
						break;
					default:
						$meta[ $mappedMeta[ $name ] ] = esc_html( wp_strip_all_tags( strval( $value ) ) );
						break;
				}
			}

			// If the term didn't have a title set at meta level, we still need to run it through the format.
			if ( ! in_array( 'title', array_keys( $meta ), true ) ) {
				$meta['title'] = $this->getTitleValue( $term );
			}

			$aioseoTerm = Models\Term::getTerm( $term->term_id );
			$aioseoTerm->set( $meta );
			$aioseoTerm->save();
		}

		if ( $this->videoSitemap ) {
			$term = get_term( $term->term_id );
			$this->videoSitemap->scanTerm( $term );
		}

		if ( count( $excludedTerms ) ) {
			$deprecatedOptions = aioseo()->internalOptions->internal->deprecatedOptions;
			if ( ! in_array( 'excludeTerms', $deprecatedOptions, true ) ) {
				array_push( $deprecatedOptions, 'excludeTerms' );
				aioseo()->internalOptions->internal->deprecatedOptions = $deprecatedOptions;
			}

			$terms = aioseo()->options->deprecated->searchAppearance->advanced->excludeTerms;
			foreach ( $sitemapExcludedTerms as $id ) {
				if ( ! intval( $id ) ) {
					continue;
				}
				$term = get_term( $id );
				if ( ! is_object( $term ) ) {
					continue;
				}
				$excludedTerm        = new \stdClass();
				$excludedTerm->type  = $term->taxonomy;
				$excludedTerm->value = $term->term_id;
				$excludedTerm->label = $term->name;
				$excludedTerm->link  = get_term_link( $term, $term->taxonomy );

				$terms[] = wp_json_encode( $excludedTerm );
			}
			aioseo()->options->deprecated->searchAppearance->advanced->excludeTerms = $terms;
		}

		if ( count( $sitemapExcludedTerms ) ) {
			aioseo()->options->sitemap->general->advancedSettings->enable = true;

			$terms = aioseo()->options->sitemap->general->advancedSettings->excludeTerms;
			foreach ( $sitemapExcludedTerms as $id ) {
				if ( ! intval( $id ) ) {
					continue;
				}
				$term = get_term( $id );
				if ( ! is_object( $term ) ) {
					continue;
				}
				$excludedTerm        = new \stdClass();
				$excludedTerm->type  = $term->taxonomy;
				$excludedTerm->value = $term->term_id;
				$excludedTerm->label = $term->name;
				$excludedTerm->link  = get_term_link( $term, $term->taxonomy );

				$terms[] = wp_json_encode( $excludedTerm );
			}
			aioseo()->options->sitemap->general->advancedSettings->excludeTerms = $terms;
		}

		if ( count( $termsToMigrate ) === $termsPerAction ) {
			try {
				$actions = as_get_scheduled_actions( [
					'hook'     => 'aioseo_migrate_term_meta',
					'status'   => \ActionScheduler_Store::STATUS_PENDING,
					'per_page' => 1
				]);

				if ( $actions ) {
					return;
				}
				as_schedule_single_action( time(), 'aioseo_migrate_term_meta' );
			} catch ( \RuntimeException $e ) {
				// Do nothing.
			}
		} else {
			delete_transient( 'aioseo_v3_migration_in_progress_terms' );
		}
	}

	/**
	 * Returns the title as it was in V3.
	 *
	 * @since 4.0.0
	 *
	 * @param  int    $post     The term object.
	 * @param  string $seoTitle The old SEO title.
	 * @return string           The title.
	 */
	protected function getTitleValue( $term, $seoTitle = '' ) {
		if ( ! isset( $term->term_id ) ) {
			return parent::getTitleValue( $term, $seoTitle );
		}

		if ( ! is_object( $term ) ) {
			return '';
		}

		$titleFormat = '%taxonomy_title% | %site_title%';
		$taxName     = ( 'post_tag' === $term->taxonomy ) ? 'tag' : $term->taxonomy;
		if ( isset( aioseo()->migration->oldOptions[ "aiosp_${taxName}_title_format" ] ) ) {
			$titleFormat = aioseo()->migration->oldOptions[ "aiosp_${taxName}_title_format" ] ? aioseo()->migration->oldOptions[ "aiosp_${taxName}_title_format" ] : $titleFormat;
		}
		return aioseo()->migration->helpers->macrosToSmartTags( $seoTitle );
	}
}